@@ -15,7 +15,7 @@ setClass("mpfr1", ## a single Multi-precision float number
                    sign= "integer",  # signum
                    d = "integer"),   # the mantissa as a vector of (32 bit) integers
 	 validity = function(object) {
-	     gmp.numb <- .mpfr.gmp.numbbits() # 32 or 64
+	     gmp.numb <- .mpfr_gmp_numbbits() # 32 or 64
 	     if(length(pr <- object@prec) != 1 || is.na(pr) || pr < 2)
 		 "invalid 'prec' slot"
 	     else if((lex <- length(ex <- object@exp)) != 2 && gmp.numb == 64)
@@ -52,7 +52,8 @@ setClass("mpfr", ## a *vector* of "mpfr1", i.e., multi-precision float numbers
 	 contains = "list", ## of "mpfr1" entries:
 	 validity = function(object) {
 	     ## should be fast ( ==> not using	is(., "mpfr1") ) :
-	     if(all(vapply(object@.Data, class, "") == "mpfr1"))
+	     if(all(lengths(cls <- lapply(object@.Data, class)) == 1L) && 
+		all(unlist(cls) == "mpfr1"))
 		 return(TRUE)
 	     ## else
 		 "Not all components are of class 'mpfr1'"
